<?php
if (!defined('_PS_VERSION_')) {
    exit;
}

use Delo\Requirements;
use Delo\Settings;
use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

require __DIR__ . '/vendor/autoload.php';

class DeloBank extends PaymentModule
{
    const PREFIX = 'delobank_';
    const MODULE_NAME = 'DeloBank (v1.0.0)';
    protected $config_form = false;
    /** @var Settings  */
    private $settingsModel;

    public function __construct()
    {
        $this->name = 'delobank';
        $this->tab = 'payments_gateways';
        $this->categoryParentEnglishName = 'Payment';
        $this->version = '1.0.0';
        $this->author = 'DeloBank';
        $this->need_instance = 0;
        $this->bootstrap = true;

        parent::__construct();

        $this->displayName = $this->l(self::MODULE_NAME);
        $this->description = $this->l('description');
        $this->confirmUninstall = $this->l('uninstall_text');
        $this->limited_currencies = ['RUR'];
        $this->ps_versions_compliancy = ['min' => '1.4', 'max' => _PS_VERSION_];
        $this->limited_countries = ['RU'];
        $this->limited_currencies = ['RUB'];
        $this->settingsModel = new Settings();
    }

    /**
     * Установка модуля
     * @return bool
     * @throws PrestaShopException
     */
    public function install()
    {
        $requirements = new Requirements();
        if (!$requirements->check()) {
            $this->_errors = $requirements->getErrors();
        }
        if (Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }
        // Создаем настройки в БД
        $settings = $this->settingsModel->settingFields();
        $defaultSettingValues = $this->settingsModel->defaultValues();
        foreach ($settings as $fieldName => $setting) {
            Configuration::updateValue(self::PREFIX . $fieldName, $defaultSettingValues[$fieldName]);
        }
        // Таблица под связь id заказа - id транзакции оплаты
        // include(__DIR__ . '/sql/install.php');
        return parent::install() &&
            $this->registerHook('payment') &&
            $this->registerHook('paymentOptions');
    }

    /**
     * Удаление модуля
     * @return bool
     */
    public function uninstall()
    {
        // Удаляем настройки
        $settings = $this->settingsModel->settingFields();
        $fields = array_keys($settings);
        foreach ($fields as $fieldName) {
            Configuration::deleteByName(self::PREFIX . $fieldName);
        }
        // Таблица под связь id заказа - id транзакции оплаты
        // include(__DIR__ . '/sql/uninstall.php');
        return parent::uninstall();
    }

    /**
     * Load the configuration form
     */
    public function getContent()
    {
        /**
         * If values have been submitted in the form, process.
         */
        if (((bool)Tools::isSubmit('submitDeloBankModule')) == true) {
            $this->postProcess();
        }
        $this->context->smarty->assign('module_dir', $this->_path);
        return $this->renderForm();
    }

    /**
     * Create the form that will be displayed in the configuration of your module.
     */
    protected function renderForm()
    {
        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $helper->default_form_language = $this->context->language->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitDeloBankModule';
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages'    => $this->context->controller->getLanguages(),
            'id_language'  => $this->context->language->id,
        ];

        return $helper->generateForm([$this->getConfigForm()]);
    }

    /**
     * Create the structure of your form.
     */
    protected function getConfigForm()
    {
        $settings = $this->settingsModel->settingFields();
        $input = [];
        $statuses = OrderState::getOrderStates((int) $this->context->language->id);
        foreach ($settings as $fieldName => $setting) {
            $newInput = [
                'type'     => $setting['type'],
                'label'    => $this->l($fieldName . '_label'),
                'name'     => self::PREFIX . $fieldName,
                'is_bool'  => $setting['type'] == 'switch' ? true : false,
                'required' => in_array('required', $setting['validate']) ? true : false,
            ];
            $hintField = $this->l($fieldName . '_label_hint');
            if ($hintField !== $fieldName . '_label_hint') {
                $newInput['desc'] = $hintField;
            }
            if ($setting['type'] == 'switch') {
                $newInput['values'] = [
                    [
                        'id'    => self::PREFIX . $fieldName . '_on',
                        'value' => 1,
                        'label' => $this->l('Enabled')
                    ],
                    [
                        'id'    => self::PREFIX . $fieldName . '_off',
                        'value' => 0,
                        'label' => $this->l('Disabled')
                    ]
                ];
            } elseif ($setting['type'] == 'select') {
                $newInput['options'] = [
                    'query' => [],
                    'id'    => 'id',
                    'name'  => 'value',
                ];
                if (!empty($setting['values'])) {
                    foreach ($setting['values'] as $k => $v) {
                        $newInput['options']['query'][] = [
                            'id'    => $k,
                            'value' => $v,
                        ];
                    }
                } elseif ($fieldName == 'order_status_before' || $fieldName == 'order_status_after' || $fieldName = 'order_status_failed') {
                    // Статусы заказа
                    foreach ($statuses as $status) {
                        $newInput['options']['query'][] = [
                            'id'    => $status['id_order_state'],
                            'value' => $status['name'],
                        ];
                    }
                }
            }
            $input[] = $newInput;
        }
        return [
            'form' => [
                'legend' => [
                    'title' => $this->l('Settings'),
                    'icon' => 'icon-cogs',
                ],
                'input' => $input,
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];
    }

    /**
     * Set values for the inputs.
     */
    protected function getConfigFormValues()
    {
        $settings = $this->settingsModel->settingFields();
        $defaultValues = $this->settingsModel->defaultValues();
        $values = [];
        foreach ($settings as $fieldName => $setting) {
            $values[self::PREFIX . $fieldName] = Configuration::get(self::PREFIX . $fieldName, $defaultValues[$fieldName]);
        }
        return $values;
    }

    /**
     * Save form data.
     */
    protected function postProcess()
    {
        $form_values = $this->getConfigFormValues();

        foreach (array_keys($form_values) as $key) {
            Configuration::updateValue($key, Tools::getValue($key));
        }
    }

    /**
     * Add the CSS & JavaScript files you want to be added on the FO.
     */
    public function hookHeader()
    {
        $this->context->controller->addJS($this->_path.'/views/js/front.js');
        $this->context->controller->addCSS($this->_path.'/views/css/front.css');
    }

    /**
     * This method is used to render the payment button,
     * Take care if the button should be displayed or not.
     */
    public function hookPayment($params)
    {
        $currency_id = $params['cart']->id_currency;
        $currency = new Currency((int)$currency_id);

        if (in_array($currency->iso_code, $this->limited_currencies) == false)
            return false;

        $this->smarty->assign('module_dir', $this->_path);

        return $this->display(__FILE__, 'views/templates/hook/payment.tpl');
    }

    /**
     * Return payment options available for PS 1.7+
     *
     * @param array Hook parameters
     * @return array|null
     */
    public function hookPaymentOptions($params)
    {
        if (!$this->active) {
            return;
        }
        $enablePaymentMethod = Configuration::get(self::PREFIX . 'status', 0);
        if (!$enablePaymentMethod) {
            return;
        }
        if (!$this->checkCurrency($params['cart'])) {
            return;
        }
        $option = new PaymentOption();
        $option->setCallToActionText(Configuration::get(self::PREFIX . 'title'))
            ->setAction($this->context->link->getModuleLink($this->name, 'validation', [], true))
            ->setInputs([[
                'name' => 'test1',
                'type' => 'hidden',
                'value' => '1',
            ]]);
        $info = Configuration::get(self::PREFIX . 'info');
        if (!empty($info)) {
            $option->setAdditionalInformation('<p>' . $info . '</p>');
        }
        return [
            $option
        ];
    }

    public function checkCurrency($cart)
    {
        $currency_order = new Currency($cart->id_currency);
        $currencies_module = $this->getCurrency($cart->id_currency);
        if (is_array($currencies_module)) {
            foreach ($currencies_module as $currency_module) {
                if ($currency_order->id == $currency_module['id_currency']) {
                    return true;
                }
            }
        }
        return false;
    }
}
